@@ -20,7 +20,6 @@ static int wacom_penpartner_irq(struct wacom_wac *wacom, void *wcombo)
 
 	switch (data[0]) {
 		case 1:
-			wacom_input_regs(wcombo);
 			if (data[5] & 0x80) {
 				wacom->tool[0] = (data[5] & 0x20) ? BTN_TOOL_RUBBER : BTN_TOOL_PEN;
 				wacom->id[0] = (data[5] & 0x20) ? ERASER_DEVICE_ID : STYLUS_DEVICE_ID;
@@ -28,7 +27,8 @@ static int wacom_penpartner_irq(struct wacom_wac *wacom, void *wcombo)
 				wacom_report_abs(wcombo, ABS_MISC, wacom->id[0]); /* report tool id */
 				wacom_report_abs(wcombo, ABS_X, wacom_le16_to_cpu(&data[1]));
 				wacom_report_abs(wcombo, ABS_Y, wacom_le16_to_cpu(&data[3]));
-				wacom_report_abs(wcombo, ABS_PRESSURE, (signed char)data[6] + 127);
+				wacom_report_abs(wcombo, ABS_PRESSURE, ((signed char)data[6] + 127));
+				wacom_report_key(wcombo, BTN_TOUCH, ((signed char)data[6] + 127));
 				wacom_report_key(wcombo, BTN_STYLUS, (data[5] & 0x40));
 			} else {
 				wacom_report_key(wcombo, wacom->tool[0], 0);
@@ -37,7 +37,6 @@ static int wacom_penpartner_irq(struct wacom_wac *wacom, void *wcombo)
 			}
 			break;
 		case 2:
-			wacom_input_regs(wcombo);
 			wacom_report_key(wcombo, BTN_TOOL_PEN, 1);
 			wacom_report_abs(wcombo, ABS_MISC, STYLUS_DEVICE_ID); /* report tool id */
 			wacom_report_abs(wcombo, ABS_X, wacom_le16_to_cpu(&data[1]));
@@ -52,7 +51,7 @@ static int wacom_penpartner_irq(struct wacom_wac *wacom, void *wcombo)
 	return 1;
 }
 
-static int wacom_pl_irq(struct wacom_wac *wacom, void *wcombo) 
+static int wacom_pl_irq(struct wacom_wac *wacom, void *wcombo)
 {
 	unsigned char *data = wacom->data;
 	int prox, pressure;
@@ -64,8 +63,6 @@ static int wacom_pl_irq(struct wacom_wac *wacom, void *wcombo)
 
 	prox = data[1] & 0x40;
 
-	wacom_input_regs(wcombo);
-
 	wacom->id[0] = ERASER_DEVICE_ID;
 	if (prox) {
 
@@ -107,7 +104,7 @@ static int wacom_pl_irq(struct wacom_wac *wacom, void *wcombo)
 		wacom_report_abs(wcombo, ABS_X, data[3] | (data[2] << 7) | ((data[1] & 0x03) << 14));
 		wacom_report_abs(wcombo, ABS_Y, data[6] | (data[5] << 7) | ((data[4] & 0x03) << 14));
 		wacom_report_abs(wcombo, ABS_PRESSURE, pressure);
-
+		wacom_report_key(wcombo, BTN_TOUCH, pressure);
 		wacom_report_key(wcombo, BTN_STYLUS, data[4] & 0x10);
 		/* Only allow the stylus2 button to be reported for the pen tool. */
 		wacom_report_key(wcombo, BTN_STYLUS2, (wacom->tool[1] == BTN_TOOL_PEN) && (data[4] & 0x20));
@@ -133,7 +130,6 @@ static int wacom_ptu_irq(struct wacom_wac *wacom, void *wcombo)
 		return 0;
 	}
 
-	wacom_input_regs(wcombo);
 	if (data[1] & 0x04) {
 		wacom_report_key(wcombo, BTN_TOOL_RUBBER, data[1] & 0x20);
 		wacom->id[0] = ERASER_DEVICE_ID;
@@ -145,6 +141,7 @@ static int wacom_ptu_irq(struct wacom_wac *wacom, void *wcombo)
 	wacom_report_abs(wcombo, ABS_X, wacom_le16_to_cpu(&data[2]));
 	wacom_report_abs(wcombo, ABS_Y, wacom_le16_to_cpu(&data[4]));
 	wacom_report_abs(wcombo, ABS_PRESSURE, wacom_le16_to_cpu(&data[6]));
+	wacom_report_key(wcombo, BTN_TOUCH, wacom_le16_to_cpu(&data[6]));
 	wacom_report_key(wcombo, BTN_STYLUS, data[1] & 0x02);
 	wacom_report_key(wcombo, BTN_STYLUS2, data[1] & 0x10);
 	return 1;
@@ -160,9 +157,7 @@ static int wacom_graphire_irq(struct wacom_wac *wacom, void *wcombo)
 		return 0;
 	}
 
-	wacom_input_regs(wcombo);
-
-	if (data[1] & 0x80) {
+	if (data[1] & 0x80) { 
 		/* in prox and not a pad data */
 
 		switch ((data[1] >> 5) & 3) {
@@ -205,6 +200,7 @@ static int wacom_graphire_irq(struct wacom_wac *wacom, void *wcombo)
 		wacom_report_abs(wcombo, ABS_Y, y);
 		if (wacom->tool[0] != BTN_TOOL_MOUSE) {
 			wacom_report_abs(wcombo, ABS_PRESSURE, data[6] | ((data[7] & 0x01) << 8));
+			wacom_report_key(wcombo, BTN_TOUCH, data[6] | ((data[7] & 0x01) << 8));
 			wacom_report_key(wcombo, BTN_STYLUS, data[1] & 0x02);
 			wacom_report_key(wcombo, BTN_STYLUS2, data[1] & 0x04);
 		}
@@ -219,6 +215,7 @@ static int wacom_graphire_irq(struct wacom_wac *wacom, void *wcombo)
 			wacom_report_abs(wcombo, ABS_DISTANCE, 0);
 		} else {
 			wacom_report_abs(wcombo, ABS_PRESSURE, 0);
+			wacom_report_key(wcombo, BTN_TOUCH, 0);
 			wacom_report_key(wcombo, BTN_STYLUS, 0);
 			wacom_report_key(wcombo, BTN_STYLUS2, 0);
 		}
@@ -361,11 +358,13 @@ static int wacom_intuos_inout(struct wacom_wac *wacom, void *wcombo)
 			wacom_report_abs(wcombo, ABS_RZ, 0);
  		} else {
 			wacom_report_abs(wcombo, ABS_PRESSURE, 0);
+			wacom_report_key(wcombo, BTN_TOUCH, 0);
 			wacom_report_abs(wcombo, ABS_TILT_X, 0);
 			wacom_report_abs(wcombo, ABS_TILT_Y, 0);
 			wacom_report_key(wcombo, BTN_STYLUS, 0);
 			wacom_report_key(wcombo, BTN_STYLUS2, 0);
 			wacom_report_abs(wcombo, ABS_WHEEL, 0);
+			wacom_report_abs(wcombo, ABS_Z, 0);
 		}
 		wacom_report_key(wcombo, wacom->tool[idx], 0);
 		wacom_report_abs(wcombo, ABS_MISC, 0); /* reset tool id */
@@ -384,6 +383,7 @@ static void wacom_intuos_general(struct wacom_wac *wacom, void *wcombo)
 	if ((data[1] & 0xb8) == 0xa0) {
 		t = (data[6] << 2) | ((data[7] >> 6) & 3);
 		wacom_report_abs(wcombo, ABS_PRESSURE, t);
+		wacom_report_key(wcombo, BTN_TOUCH, t);
 		wacom_report_abs(wcombo, ABS_TILT_X,
 				((data[7] << 1) & 0x7e) | (data[8] >> 7));
 		wacom_report_abs(wcombo, ABS_TILT_Y, data[8] & 0x7f);
@@ -413,12 +413,10 @@ static int wacom_intuos_irq(struct wacom_wac *wacom, void *wcombo)
                 return 0;
 	}
 
-	wacom_input_regs(wcombo);
-
 	/* tool number */
 	idx = data[1] & 0x01;
 
-	/* pad packets. Works as a second tool and is always in prox */
+	/* pad packets. Works as a second tool */
 	if (data[0] == 12) {
 		/* initiate the pad as a device */
 		if (wacom->tool[1] != BTN_TOOL_FINGER)
@@ -437,12 +435,14 @@ static int wacom_intuos_irq(struct wacom_wac *wacom, void *wcombo)
 		wacom_report_abs(wcombo, ABS_RX, ((data[1] & 0x1f) << 8) | data[2]);
 		wacom_report_abs(wcombo, ABS_RY, ((data[3] & 0x1f) << 8) | data[4]);
 
-		if ((data[5] & 0x1f) | (data[6] & 0x1f) | (data[1] & 0x1f) | 
-			data[2] | (data[3] & 0x1f) | data[4])
+		if ((data[5] & 0x1f) | (data[6] & 0x1f) | (data[1] & 0x1f) |
+			data[2] | (data[3] & 0x1f) | data[4]) {
 			wacom_report_key(wcombo, wacom->tool[1], 1);
-		else
+			wacom_report_abs(wcombo, ABS_MISC, PAD_DEVICE_ID);
+		} else {
 			wacom_report_key(wcombo, wacom->tool[1], 0);
-		wacom_report_abs(wcombo, ABS_MISC, PAD_DEVICE_ID); 
+			wacom_report_abs(wcombo, ABS_MISC, 0); 
+		}
 		wacom_input_event(wcombo, EV_MSC, MSC_SERIAL, 0xffffffff);
                 return 1;
 	}
@@ -527,49 +527,69 @@ static int wacom_intuos_irq(struct wacom_wac *wacom, void *wcombo)
 			wacom_report_key(wcombo, BTN_EXTRA,  data[8] & 0x08);
 		}
 	}
-	wacom_report_abs(wcombo, ABS_MISC, wacom->id[idx]);
+
+	wacom_report_abs(wcombo, ABS_MISC, wacom->id[idx]); /* report tool id */
 	wacom_report_key(wcombo, wacom->tool[idx], 1);
 	wacom_input_event(wcombo, EV_MSC, MSC_SERIAL, wacom->serial[idx]);
 	return 1;
 }
 
-int wacom_tpc_irq (struct wacom_wac *wacom, void *wcombo)
+int wacom_tpc_irq(struct wacom_wac *wacom, void *wcombo)
 {
-	static int stylusInProx, touchInProx;
 	char *data = wacom->data;
-	int prox = data[1] & 0x20, pressure;
+	int prox = 0, pressure;
+	static int stylusInProx, touchInProx = 1, touchOut;
 	struct urb *urb = ((struct wacom_combo *)wcombo)->urb;
 
-	if (urb->actual_length == 5 || data[0] == 6) { /* Touch data */
-		if (stylusInProx) { /* stylus is still in prox */
-			touchInProx = 1;
-		} else {
-			if (data[0] & 0x03) {
-				if (!touchInProx) {
-					wacom->tool[1] = BTN_TOUCH;
+	dbg("wacom_tpc_irq: received report #%d", data[0]);
+
+	if (urb->actual_length == 5 || data[0] == 6) { /* Touch data */	
+		if (urb->actual_length == 5) {  /* with touch */
+			prox = data[0] & 0x03;
+		} else {  /* with capacity */
+			prox = data[1] & 0x03;
+		}
+
+		if (!stylusInProx) { /* stylus not in prox */
+			if (prox) { 
+				if (touchInProx) {
+					wacom->tool[1] = BTN_TOOL_DOUBLETAP;
 					wacom->id[0] = TOUCH_DEVICE_ID;
-					wacom_report_abs(wcombo, ABS_X, wacom_le16_to_cpu(&data[1]));
-					wacom_report_abs(wcombo, ABS_Y, wacom_le16_to_cpu(&data[3]));
+					if (urb->actual_length != 5) {
+						wacom_report_abs(wcombo, ABS_X, wacom_le16_to_cpu(&data[2]));
+						wacom_report_abs(wcombo, ABS_Y, wacom_le16_to_cpu(&data[4]));
+						wacom_report_abs(wcombo, ABS_PRESSURE, wacom_le16_to_cpu(&data[6]));
+						wacom_report_key(wcombo, BTN_TOUCH, wacom_le16_to_cpu(&data[6]));
+					} else {
+						wacom_report_abs(wcombo, ABS_X, wacom_le16_to_cpu(&data[1]));
+						wacom_report_abs(wcombo, ABS_Y, wacom_le16_to_cpu(&data[3]));
+						wacom_report_key(wcombo, BTN_TOUCH, 1);
+					}
 					wacom_report_abs(wcombo, ABS_MISC, wacom->id[0]);
-					wacom_report_key(wcombo, wacom->tool[1], data[0] & 0x01);
+					wacom_report_key(wcombo, wacom->tool[1], prox & 0x01);
+					touchOut = 1;
 					return 1;
 				}
 			} else {
-				wacom_report_abs(wcombo, ABS_MISC, 0);
-				wacom_report_key(wcombo, wacom->tool[1], 0);
-				touchInProx = 0;
+				wacom_report_abs(wcombo, ABS_MISC, wacom->id[0]);
+				wacom_report_key(wcombo, wacom->tool[1], prox & 0x01);
+				wacom_report_key(wcombo, BTN_TOUCH, 0);
+				touchOut = 0;
+				touchInProx = 1;
 				return 1;
 			}
-		}
-	} else if (data[0] == 2) { /* Penabled */
-		/* touch was in control */
-		if (wacom->id[0] == TOUCH_DEVICE_ID) { 
-			/* let it go */
-			wacom_report_abs(wcombo, ABS_MISC, wacom->id[0]);
-			wacom_report_key(wcombo, wacom->tool[1], 0);
+		} else if (touchOut || !prox) { /* force touch out-prox */
+			wacom_report_abs(wcombo, ABS_MISC, TOUCH_DEVICE_ID);
+			wacom_report_key(wcombo, BTN_TOUCH, 0);
+			touchOut = 0;
+			touchInProx = 1;
 			return 1;
 		}
-		
+	} else if (data[0] == 2) { /* Penabled */
+		prox = data[1] & 0x20;
+
+		touchInProx = 0;
+
 		wacom->id[0] = ERASER_DEVICE_ID;
 
 		/*
@@ -589,7 +609,7 @@ int wacom_tpc_irq (struct wacom_wac *wacom, void *wcombo)
 				 * was entered with stylus2 pressed 
 				 * report out proximity for previous tool 
 				*/
-				wacom_report_abs(wcombo, ABS_MISC, 0);
+				wacom_report_abs(wcombo, ABS_MISC, wacom->id[0]);
 				wacom_report_key(wcombo, wacom->tool[1], 0);
 				wacom_input_sync(wcombo);
 
@@ -611,8 +631,10 @@ int wacom_tpc_irq (struct wacom_wac *wacom, void *wcombo)
 			if (pressure < 0) 
 				pressure = wacom->features->pressure_max + pressure + 1;
 			wacom_report_abs(wcombo, ABS_PRESSURE, pressure);
+			wacom_report_key(wcombo, BTN_TOUCH, pressure);
 		} else {
 			wacom_report_abs(wcombo, ABS_PRESSURE, 0);
+			wacom_report_key(wcombo, BTN_TOUCH, 0);
 			wacom_report_key(wcombo, BTN_STYLUS, 0);
 			wacom_report_key(wcombo, BTN_STYLUS2, 0);
 		}
@@ -634,9 +656,9 @@ int wacom_wac_irq(struct wacom_wac *wacom_wac, void *wcombo)
 		case PL:
 			return (wacom_pl_irq(wacom_wac, wcombo));
 			break;
-		case WACOM_MO:
 		case WACOM_G4:
 		case GRAPHIRE:
+		case WACOM_MO:
 			return (wacom_graphire_irq(wacom_wac, wcombo));
 			break;
 		case PTU:
@@ -679,6 +701,7 @@ void wacom_init_input_dev(struct input_dev *input_dev, struct wacom_wac *wacom_w
 			/* fall through */
 		case INTUOS3S:
 			input_dev_i3s(input_dev, wacom_wac);
+			/* fall through */
 		case INTUOS:
 			input_dev_i(input_dev, wacom_wac);
 			break;
