@@ -1,5 +1,5 @@
 /*
- * drivers/usb/input/wacom_sys.c
+ * drivers/input/tablet/wacom_sys.c
  *
  *  USB Wacom Graphire and Wacom Intuos tablet support - system specific code
  */
@@ -17,6 +17,7 @@
 /* defines to get HID report descriptor */
 #define HID_DEVICET_HID		(USB_TYPE_CLASS | 0x01)
 #define HID_DEVICET_REPORT	(USB_TYPE_CLASS | 0x02)
+#define HID_USAGE_UNDEFINED		0x00
 #define HID_USAGE_PAGE			0x05
 #define HID_USAGE_PAGE_DIGITIZER	0x0d
 #define HID_USAGE_PAGE_DESKTOP		0x01
@@ -77,7 +78,7 @@ static struct input_dev * get_input_dev(struct wacom_combo *wcombo)
 	return wcombo->wacom->dev;
 }
 
-static void wacom_sys_irq(struct urb *urb, struct pt_regs *regs)
+static void wacom_sys_irq(struct urb *urb)
 {
 	struct wacom *wacom = urb->context;
 	struct wacom_combo wcombo;
@@ -100,8 +101,7 @@ static void wacom_sys_irq(struct urb *urb, struct pt_regs *regs)
 
 	wcombo.wacom = wacom;
 	wcombo.urb = urb;
-	wcombo.regs = regs;
-	
+
 	if (wacom_wac_irq(wacom->wacom_wac, (void *)&wcombo))
 		input_sync(get_input_dev(&wcombo));
 
@@ -150,12 +150,6 @@ __u16 wacom_le16_to_cpu(unsigned char *data)
 	return value;
 }
 
-void wacom_input_regs(void *wcombo)
-{
-	input_regs(get_input_dev((struct wacom_combo *)wcombo), ((struct wacom_combo *)wcombo)->regs);
-	return;
-}
-
 void wacom_input_sync(void *wcombo)
 {
 	input_sync(get_input_dev((struct wacom_combo *)wcombo));
@@ -164,7 +158,7 @@ void wacom_input_sync(void *wcombo)
 
 static int wacom_open(struct input_dev *dev)
 {
-	struct wacom *wacom = dev->private;
+	struct wacom *wacom = input_get_drvdata(dev);
 
 	wacom->irq->dev = wacom->usbdev;
 	if (usb_submit_urb(wacom->irq, GFP_KERNEL))
@@ -175,7 +169,7 @@ static int wacom_open(struct input_dev *dev)
 
 static void wacom_close(struct input_dev *dev)
 {
-	struct wacom *wacom = dev->private;
+	struct wacom *wacom = input_get_drvdata(dev);
 
 	usb_kill_urb(wacom->irq);
 }
@@ -248,6 +242,7 @@ void input_dev_pt(struct input_dev *input_dev, struct wacom_wac *wacom_wac)
 	input_dev->keybit[LONG(BTN_DIGI)] |= BIT(BTN_TOOL_RUBBER);
 }
 
+
 static void wacom_paser_hid(struct usb_interface *intf, struct hid_descriptor *hid_desc, 
 		struct wacom_wac *wacom_wac, char *report)
 {
@@ -278,12 +273,6 @@ static void wacom_paser_hid(struct usb_interface *intf, struct hid_descriptor *h
 				usage = WCM_DESKTOP;
 				i++;
 				continue;
-			    case HID_USAGE_PAGE_VDEFINED:
-				if (!report[i+3]) {  /* capacity */
-					wacom_wac->features->pressure_max = (unsigned short)report[i+5];
-				}
-				i += 6;
-				continue;
 			}
 		}
 
@@ -327,6 +316,13 @@ static void wacom_paser_hid(struct usb_interface *intf, struct hid_descriptor *h
 				pen = 1;
 				i++;
 				break;
+			    case HID_USAGE_UNDEFINED:
+				if (usage == WCM_DESKTOP && finger) { /* capacity */
+					wacom_wac->features->pressure_max = (unsigned short)
+						(wacom_le16_to_cpu(&report[i+3]));
+				}
+				i += 4;
+				break;
 			}
 		}
 
@@ -339,12 +335,15 @@ static void wacom_paser_hid(struct usb_interface *intf, struct hid_descriptor *h
 
 static int wacom_probe(struct usb_interface *intf, const struct usb_device_id *id)
 {
+	int error = -ENOMEM;
 	struct usb_device *dev = interface_to_usbdev(intf);
+	struct usb_host_interface *interface = intf->cur_altsetting;
 	struct usb_endpoint_descriptor *endpoint;
 	struct wacom *wacom;
 	struct wacom_wac *wacom_wac;
 	struct input_dev *input_dev;
-	char rep_data[2], limit = 0, mode = 2;
+	char rep_data[2], limit = 0, mode = 2, *report = NULL;
+	struct hid_descriptor *hid_desc;
 
 	wacom = kzalloc(sizeof(struct wacom), GFP_KERNEL);
 	wacom_wac = kzalloc(sizeof(struct wacom_wac), GFP_KERNEL);
@@ -372,8 +371,8 @@ static int wacom_probe(struct usb_interface *intf, const struct usb_device_id *i
 	wacom->wacom_wac = wacom_wac;
 	usb_to_input_id(dev, &input_dev->id);
 
-	input_dev->cdev.dev = &intf->dev;
-	input_dev->private = wacom;
+	input_dev->dev.parent = &intf->dev;
+	input_set_drvdata(input_dev, wacom);
 	input_dev->open = wacom_open;
 	input_dev->close = wacom_close;
 
@@ -396,11 +395,11 @@ static int wacom_probe(struct usb_interface *intf, const struct usb_device_id *i
 	}
 
 	input_dev->evbit[0] |= BIT(EV_KEY) | BIT(EV_ABS);
-	input_dev->keybit[LONG(BTN_DIGI)] |= BIT(BTN_TOOL_PEN) | BIT(BTN_STYLUS);
+	input_dev->keybit[LONG(BTN_DIGI)] |= BIT(BTN_TOOL_PEN) | BIT(BTN_STYLUS) | BIT(BTN_TOUCH);
 	input_set_abs_params(input_dev, ABS_X, 0, wacom_wac->features->x_max, 4, 0);
 	input_set_abs_params(input_dev, ABS_Y, 0, wacom_wac->features->y_max, 4, 0);
 	if (wacom_wac->features->type == TABLETPC) {
-		input_dev->keybit[LONG(BTN_DIGI)] |= BIT(BTN_TOUCH);
+		input_dev->keybit[LONG(BTN_DIGI)] |= BIT(BTN_TOOL_DOUBLETAP);
 		input_set_abs_params(input_dev, ABS_RX, 0, wacom_wac->features->touch_x_max, 4, 0);
 		input_set_abs_params(input_dev, ABS_RY, 0, wacom_wac->features->touch_y_max, 4, 0);
 	}
@@ -416,7 +415,9 @@ static int wacom_probe(struct usb_interface *intf, const struct usb_device_id *i
 	wacom->irq->transfer_dma = wacom->data_dma;
 	wacom->irq->transfer_flags |= URB_NO_TRANSFER_DMA_MAP;
 
-	input_register_device(wacom->dev);
+	error = input_register_device(wacom->dev);
+	if (error)
+		goto fail3;
 
 	/* Ask the tablet to report tablet data if it is not a Tablet PC. 
 	 * Repeat until it succeeds 
@@ -431,13 +432,15 @@ static int wacom_probe(struct usb_interface *intf, const struct usb_device_id *i
 	}
 
 	usb_set_intfdata(intf, wacom);
+	kfree(report);
 	return 0;
 
-fail2:	usb_buffer_free(dev, 10, wacom_wac->data, wacom->data_dma);
-fail1:	input_free_device(input_dev);
+ fail3:	usb_free_urb(wacom->irq);
+ fail2:	usb_buffer_free(dev, 10, wacom_wac->data, wacom->data_dma);
+ fail1:	input_free_device(input_dev);
 	kfree(wacom);
 	kfree(wacom_wac);
-	return -ENOMEM;
+	return error;
 }
 
 static void wacom_disconnect(struct usb_interface *intf)
@@ -456,7 +459,6 @@ static void wacom_disconnect(struct usb_interface *intf)
 }
 
 static struct usb_driver wacom_driver = {
-	.owner =        THIS_MODULE,
 	.name =		"wacom",
 	.probe =	wacom_probe,
 	.disconnect =	wacom_disconnect,
