@@ -21,6 +21,7 @@
 **   2007-02-06 0.0.1-pc - Support keystrokes
 **   2008-01-17 0.0.2-pc - Add Display Toggle
 **   2008-08-01 0.0.3-pc - Merge patch 1998051 (Yuri Shchedov)
+**   2008-12-10 0.0.4-pc - Updated patch 1998051 for none KP buttons
 */
 
 /* This pseudo-header file is included both from the X11 driver, and from
@@ -78,30 +79,41 @@ static ACTIONCODE specific_code [] =
 	{ "Pause",	    XK_Pause },
 	{ "ScrollLock",	    XK_Scroll_Lock },
 	{ "SysReq",	    XK_Sys_Req },
-	{ "Home",	    XK_KP_Home },
-	{ "PgUp",	    XK_KP_Page_Up },
-	{ "PageUp",	    XK_KP_Page_Up },
-	{ "PgDn",	    XK_KP_Page_Down },
-	{ "PageDown",	    XK_KP_Page_Down },
-	{ "End",	    XK_KP_End },
-	{ "Insert",	    XK_KP_Insert },
-	{ "Ins",	    XK_KP_Insert },
-	{ "Delete",	    XK_KP_Delete },
-	{ "Del",	    XK_KP_Delete },
-	{ "Left",	    XK_KP_Left },
-	{ "Up",		    XK_KP_Up },
-	{ "Down",	    XK_KP_Down },
-	{ "Right",	    XK_KP_Right },
+	{ "Home",	    XK_Home },
+	{ "PageUp",	    XK_Page_Up },
+	{ "PageDown",	    XK_Page_Down },
+	{ "End",	    XK_End },
+	{ "Insert",	    XK_Insert },
+	{ "Delete",	    XK_Delete },
+	{ "Left",	    XK_Left },
+	{ "Up",		    XK_Up },
+	{ "Down",	    XK_Down },
+	{ "Right",	    XK_Right },
 	{ "BackSpace",	    XK_BackSpace },
 	{ "Enter",	    XK_Return },
 	{ "NumLock",	    XK_Num_Lock },
 	{ "space",	    XK_space },
 	{ "quotedbl",       XK_quotedbl },
 	{ "backslash",      XK_backslash },
-	{ "NumpadPlus",	    XK_KP_Add },
+	{ "KPHome",	    XK_KP_Home },
+	{ "PgUp",	    XK_KP_Page_Up },
+	{ "PgDn",	    XK_KP_Page_Down },
+	{ "KPEnd",	    XK_KP_End },
+	{ "Ins",	    XK_KP_Insert },
+	{ "Del",	    XK_KP_Delete },
+	{ "KPLeft",	    XK_KP_Left },
+	{ "KPUp",	    XK_KP_Up },
+	{ "KPDown",	    XK_KP_Down },
+	{ "KPRight",	    XK_KP_Right },
+	{ "KPEnter",	    XK_KP_Enter },
+	{ "Plus",	    XK_KP_Add },
+	{ "Minus",	    XK_KP_Subtract},
+	{ "Divide",	    XK_KP_Divide },
+	{ "Multiply",	    XK_KP_Multiply },
+	{ "NumpadPlus",     XK_KP_Add },
 	{ "NumpadMinus",    XK_KP_Subtract},
 	{ "NumpadDivide",   XK_KP_Divide },
-	{ "NumpadMultiply", XK_KP_Multiply },
+        { "NumpadMultiply", XK_KP_Multiply },
 	{ "break",	    XK_Break },
 	{ "print",	    XK_Print }
 };
