@@ -1,6 +1,6 @@
 /*
  * Copyright 1995-2002 by Frederic Lepied, France. <Lepied@XFree86.org>
- * Copyright 2002-2008 by Ping Cheng, Wacom. <pingc@wacom.com>
+ * Copyright 2002-2009 by Ping Cheng, Wacom. <pingc@wacom.com>
  *                                                                            
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -139,17 +139,7 @@ LocalDevicePtr xf86WcmAllocate(char* name, int flag)
 	for (i=0; i<MAX_BUTTONS; i++)
 		priv->button[i] = IsPad (priv) ?
 			(AC_BUTTON | (MAX_MOUSE_BUTTONS/2 + i + 1)) : (AC_BUTTON | (i + 1));
-/*	for (i=MAX_MOUSE_BUTTONS/2; i<MAX_BUTTONS; i++)
-		priv->button[i] = IsPad (priv) ?
-			(AC_KEY | (XK_F1 + i)) : (AC_BUTTON | (i + 1));
-*/
-	/* Now for backward compatibility make some keys emit button events
-	 * with button indices 9-16...
-	 */
-/*	if (IsPad (priv))
-		for (i = 0; i < 8; i++)
-			priv->button[i] = (AC_BUTTON | (i + 9));
-*/
+
 	for (i=0; i<MAX_BUTTONS; i++)
 		for (j=0; j<256; j++)
 			priv->keys[i][j] = 0;
@@ -378,7 +368,7 @@ static void xf86WcmUninit(InputDriverPtr drv, LocalDevicePtr local, int flags)
     
 	DBG(1, priv->debugLevel, ErrorF("xf86WcmUninit\n"));
 
-#ifndef WCM_UNINIT_CALLED
+#ifndef WCM_XORG_XSERVER_1_4
 	gWacomModule.DevProc(local->dev, DEVICE_OFF);
 #endif
 
@@ -386,7 +376,14 @@ static void xf86WcmUninit(InputDriverPtr drv, LocalDevicePtr local, int flags)
 	if (priv->pPressCurve)
 		xfree(priv->pPressCurve);
     
+#ifndef WCM_XORG_XSERVER_1_6
+	/* don't free priv for X server 1.6 or later here
+	 * otherwise X server crashes 
+	 */
 	xfree(priv);
+	local->private = NULL;
+#endif
+
 	xf86DeleteInput(local, 0);    
 }
 
@@ -573,6 +570,8 @@ static LocalDevicePtr xf86WcmInit(InputDriverPtr drv, IDevPtr dev, int flags)
 	 */
 	if (IsPad(priv))
 		xf86WcmSetPadCoreMode(local);
+else
+xf86Msg(X_CONFIG, "%s (%s) is not a pad \n", local->name, dev->identifier);
 
 	/* Store original local Core flag so it can be changed later */
 	if (local->flags & (XI86_ALWAYS_CORE | XI86_CORE_POINTER))
@@ -590,8 +589,7 @@ static LocalDevicePtr xf86WcmInit(InputDriverPtr drv, IDevPtr dev, int flags)
 		common->wcmDevCls = &gWacomISDV4Device;
 		xf86Msg(X_CONFIG, "%s: forcing TabletPC ISD V4 protocol\n",
 			dev->identifier);
-		common->wcmTPCButton = 1;	/* Tablet PC buttons on by default */
-		common->wcmTPCButtonDefault = 1;
+		common->wcmTPCButtonDefault = 1; /* Tablet PC buttons on by default */
 	}
 
 	s = xf86FindOptionValue(local->options, "Rotate");
@@ -840,7 +838,7 @@ static LocalDevicePtr xf86WcmInit(InputDriverPtr drv, IDevPtr dev, int flags)
 	common->wcmTPCButton = xf86SetBoolOption(local->options, 
 			"TPCButton", common->wcmTPCButtonDefault);
 	if ( common->wcmTPCButton )
-		xf86Msg(X_CONFIG, "%s: Tablet PC buttons on \n", common->wcmDevice);
+	xf86Msg(X_CONFIG, "%s: Tablet PC buttons are on \n", common->wcmDevice);
 
 	/* Touch applies to the whole tablet */
 	common->wcmTouch = xf86SetBoolOption(local->options, "Touch", common->wcmTouchDefault);
@@ -993,7 +991,11 @@ static XF86ModuleVersionInfo xf86WcmVersionRec =
 	MODULEVENDORSTRING,
 	MODINFOSTRING1,
 	MODINFOSTRING2,
+#ifdef WCM_XORG_XSERVER_1_6
+	XORG_VERSION_CURRENT,
+#else
 	XF86_VERSION_CURRENT,
+#endif
 	1, 0, 0,
 	ABI_CLASS_XINPUT,
 	ABI_XINPUT_VERSION,
