@@ -515,7 +515,7 @@ Bool usbWcmInit(LocalDevicePtr local, char* id, float *version)
 		/* initialize capacity parameters for touch */
 		if (common->tablet_id == 0x9A)
 		{
-			common->wcmCapacity = 0;
+			common->wcmCapacity = 3;
 			common->wcmCapacityDefault = 3; 
 		}
 		else
@@ -526,15 +526,21 @@ Bool usbWcmInit(LocalDevicePtr local, char* id, float *version)
 
 		if (common->tablet_id == 0x9A || common->tablet_id == 0x93)
 		{
-			char *s = xf86FindOptionValue(local->options, "Touch");
-			if ( !s || (strstr(s, "on")) )  /* touch option is on */
-			{
-				common->wcmTouch = 1;
-			}
-
 			/* TouchDefault was off for all devices */
-			/* defaults to enable when touch is supported */
+			/* except when touch is supported */
 			common->wcmTouchDefault = 1;
+			common->wcmTPCButtonDefault = 1; /* Tablet PC buttons on by default */
+
+			/* check if touch was turned off in xorg.conf */
+			common->wcmTouch = xf86SetBoolOption(local->options, "Touch", common->wcmTouchDefault);
+			if ( common->wcmTouch )
+				xf86Msg(X_CONFIG, "%s: Touch is enabled \n", common->wcmDevice);
+
+			/* Tablet PC button applied to the whole tablet. Not just one tool */
+			common->wcmTPCButton = xf86SetBoolOption(local->options, 
+					"TPCButton", common->wcmTPCButtonDefault);
+			if ( common->wcmTPCButton )
+				xf86Msg(X_CONFIG, "%s: Tablet PC buttons are on \n", common->wcmDevice);
 		}
 	}
 
@@ -690,10 +696,10 @@ int usbWcmGetRanges(LocalDevicePtr local)
 
 	if (IsTouch(priv) && common->wcmMaxX && common->wcmMaxY)
 	{
-		common->wcmTouchResolX = common->wcmMaxTouchX *
-			common->wcmResolX / common->wcmMaxX;
-		common->wcmTouchResolY = common->wcmMaxTouchY *
-			common->wcmResolY / common->wcmMaxY;
+		common->wcmTouchResolX = (int)((double)(common->wcmMaxTouchX *
+			common->wcmResolX) / (double)common->wcmMaxX + 0.5);
+		common->wcmTouchResolY = (int)((double)(common->wcmMaxTouchY *
+			common->wcmResolY) / (double)common->wcmMaxY + 0.5);
 		if (!common->wcmTouchResolX || !common->wcmTouchResolY)
 		{
 			ErrorF("WACOM: touch max value(s) was wrong MaxTouchY"
@@ -794,7 +800,7 @@ static void usbParseEvent(LocalDevicePtr local,
 	if (common->wcmEventCnt >=
 		(sizeof(common->wcmEvents)/sizeof(*common->wcmEvents)))
 	{
-		ErrorF("usbParse: Exceeded event queue (%d)\n",
+		ErrorF("usbParse: Exceeded event queue (%d) \n",
 				common->wcmEventCnt);
 		common->wcmEventCnt = 0;
 		common->wcmLastToolSerial = 0;
@@ -809,6 +815,11 @@ static void usbParseEvent(LocalDevicePtr local,
 	if ((event->type == EV_MSC) && (event->code == MSC_SERIAL))
 	{
 		/* save the serial number so we can look up the channel number later */
+		if (event->value == 0) /* serial number should never be 0 */
+		{
+			ErrorF("usbParse: Ignoring event from invalid serial 0\n");
+			return;
+		}
 		common->wcmLastToolSerial = event->value;
 
 		/* if SYN_REPORT is end of record indicator, we are done */
@@ -841,7 +852,7 @@ static void usbParseEvent(LocalDevicePtr local,
 	channel = -1;
 	if (common->wcmProtocolLevel == 4)
 	{
-		/* Protocol 4 don't support tool serial numbers */
+		/* Protocol 4 doesn't support tool serial numbers */
 		if (common->wcmLastToolSerial == 0xf0)
 			channel = 1;
 		else
@@ -853,7 +864,7 @@ static void usbParseEvent(LocalDevicePtr local,
 			common->wcmChannel[channel].work.proximity = 1;
 		}
 	}
-	else
+	else if (common->wcmLastToolSerial) /* serial number should never be 0 */
 	{
 		/* find existing channel */
 		for (i=0; i<MAX_CHANNELS; ++i)
@@ -889,7 +900,8 @@ static void usbParseEvent(LocalDevicePtr local,
 	{
 		/* this should never happen in normal use */
 		ErrorF("usbParse: Exceeded channel count; "
-			"ignoring.\n");
+			"ignoring the events.\n");
+		common->wcmEventCnt = 0;
 		return;
 
 	}
@@ -1016,7 +1028,7 @@ static void usbParseChannel(LocalDevicePtr local, int channel, int serial)
 				ds->device_id = PAD_DEVICE_ID;
 				ds->proximity = (event->value != 0);
 			}
-			else if (event->code == BTN_TOUCH)
+			else if (event->code == BTN_TOOL_DOUBLETAP)
 			{
 				DBG(6, common->debugLevel, ErrorF(
 					"USB Touch detected %x (value=%d)\n",
