@@ -73,9 +73,11 @@
  * 2008-07-09 47-pc0.8.1   - new release
  * 2008-07-17 47-pc0.8.1-1 - Support USB TabletPC
  * 2008-08-27 47-pc0.8.1-4 - Support Bamboo1 Meadium and Monarch
+ * 2008-11-11 47-pc0.8.2 - new release
+ * 2008-12-22 47-pc0.8.2-1 - fixed a few issues
  */
 
-static const char identification[] = "$Identification: 47-0.8.1-4 $";
+static const char identification[] = "$Identification: 47-0.8.2-1 $";
 
 /****************************************************************************/
 
@@ -531,8 +533,14 @@ static int xf86WcmRegisterX11Devices (LocalDevicePtr local)
 
 	nbaxes = priv->naxes;       /* X, Y, Pressure, Tilt-X, Tilt-Y, Wheel */
 	nbbuttons = priv->nbuttons; /* Use actual number of buttons, if possible */
-	nbkeys = nbbuttons;         /* Same number of keys since any button may be */
-	                            /* configured as an either mouse button or key */
+	nbkeys = nbbuttons;         /* Same number of keys since any button may be 
+	                             * configured as an either mouse button or key */
+
+	if (!nbbuttons)
+		nbbuttons = nbkeys = 1;	    /* Xserver 1.5 or later crashes when 
+			            	     * nbbuttons = 0 while sending a beep 
+			             	     * This is only a workaround. 
+				     	     */
 
 	DBG(10, priv->debugLevel, ErrorF("xf86WcmRegisterX11Devices "
 		"(%s) %d buttons, %d keys, %d axes\n",
@@ -569,7 +577,7 @@ static int xf86WcmRegisterX11Devices (LocalDevicePtr local)
 			return FALSE;
 	}
 
-	if (nbaxes || nbaxes > 6)
+	if (!nbaxes || nbaxes > 6)
 		nbaxes = priv->naxes = 6;
 
 	if (InitValuatorClassDeviceStruct(local->dev, nbaxes,
@@ -577,7 +585,9 @@ static int xf86WcmRegisterX11Devices (LocalDevicePtr local)
 					  xf86GetMotionEvents,
 					  local->history_size,
 #else
+#if GET_ABI_MAJOR(ABI_XINPUT_VERSION) < 3
 					  GetMotionHistory,
+#endif
 					  GetMotionHistorySize(),
 #endif
 					  ((priv->flags & ABSOLUTE_FLAG) ?
@@ -700,6 +710,12 @@ static int xf86WcmRegisterX11Devices (LocalDevicePtr local)
 		InitValuatorAxisStruct(local->dev, 5, 0, 1023, 1, 1, 1);
 	}
 
+	if (IsTouch(priv))
+	{
+		/* hard prox out */
+		priv->hardProx = 0;
+	}
+
 	return TRUE;
 }
 
@@ -751,7 +767,7 @@ char *xf86WcmEventAutoDevProbe (LocalDevicePtr local)
 		ErrorF("%s waiting 100 msec (total %dms) for device to become ready\n", local->name, wait); 
 		usleep(100*1000);
 	}
-	ErrorF("%s no synaptics event device found (checked %d nodes, waited %d msec)\n",
+	ErrorF("%s no Wacom event device found (checked %d nodes, waited %d msec)\n",
 		local->name, i + 1, wait);
 	return FALSE;
 }
@@ -849,6 +865,7 @@ void xf86WcmReadPacket(LocalDevicePtr local)
 	WacomDevicePtr priv = (WacomDevicePtr)local->private;
 	WacomCommonPtr common = priv->common;
 	int len, pos, cnt, remaining;
+	unsigned char * data;
 
  	DBG(10, common->debugLevel, ErrorF("xf86WcmReadPacket: device=%s"
 		" fd=%d \n", common->wcmDevice, local->fd));
@@ -884,7 +901,22 @@ void xf86WcmReadPacket(LocalDevicePtr local)
 
 	pos = 0;
 
-	/* while there are whole packets present, parse data */
+	/* while there are whole packets present, check the packet length
+	 * for serial ISDv4 packet since it's different for pen and touch
+	 */
+	if (common->wcmForceDevice == DEVICE_ISDV4 && common->wcmDevCls != &gWacomUSBDevice) 
+	{
+		common->wcmPktLength = 9;
+		data = common->buffer;
+		if ( data[0] & 0x18 )
+		{
+			if (common->wcmMaxCapacity)
+				common->wcmPktLength = 7;
+			else
+				common->wcmPktLength = 5;
+		}
+	}
+
 	while ((common->bufpos - pos) >=  common->wcmPktLength)
 	{
 		/* parse packet */
@@ -895,6 +927,29 @@ void xf86WcmReadPacket(LocalDevicePtr local)
 			break;
 		}
 		pos += cnt;
+
+		if (common->wcmDevCls != &gWacomUSBDevice) 
+		{
+			data = common->buffer + pos;
+			if ( data[0] & 0x18 )
+			{
+				if (common->wcmPktLength == 9)
+				{
+					DBG(1, common->debugLevel, 
+						ErrorF("xf86WcmReadPacket: not a pen data any more \n"));
+					break;	
+				}
+			}
+			else
+			{
+				if (common->wcmPktLength != 9)
+				{
+					DBG(1, common->debugLevel, 
+						ErrorF("xf86WcmReadPacket: not a touch data any more \n"));
+					break;	
+				}
+			}
+		}
 	}
  
 	if (pos)
